@@ -1,18 +1,7 @@
 /*
- * Copyright 2016 ROSA
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+    SPDX-FileCopyrightText: 2016 ROSA
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 ////////////////////////////////////////////////////////////////////////////////
 // Implementation of ImageWriter
@@ -22,10 +11,17 @@
 #include <KLocalizedString>
 
 #if defined(Q_OS_LINUX)
+#include <kauth_version.h>
+#if KAUTH_VERSION >= QT_VERSION_CHECK(5, 92, 0)
+#include <KAuth/ActionReply>
+#include <KAuth/HelperSupport>
+#else
 #include <KAuth>
 #endif
+#endif
 
 #include <QFile>
+#include <KCompressionDevice>
 
 #include "common.h"
 #include "physicaldevice.h"
@@ -80,6 +76,26 @@ void ImageWriter::writeImage()
                 throw i18n("Failed to open the image file: %1", imageFile.errorString());
         }
 
+        QIODevice* device;
+        if (imageFile.fileName().endsWith(".gz")) {
+            device = new KCompressionDevice(&imageFile, true, KCompressionDevice::GZip);
+            if (!device->open(QIODevice::ReadOnly)) {
+                throw i18n("Failed to open compression device: %1", device->errorString());
+            }
+        } else if (imageFile.fileName().endsWith(".xz")) {
+            device = new KCompressionDevice(&imageFile, true, KCompressionDevice::Xz);
+            if (!device->open(QIODevice::ReadOnly)) {
+                throw i18n("Failed to open compression device: %1", device->errorString());
+            }
+        } else if (imageFile.fileName().endsWith(".zstd")) {
+            device = new KCompressionDevice(&imageFile, true, KCompressionDevice::Zstd);
+            if (!device->open(QIODevice::ReadOnly)) {
+                throw i18n("Failed to open compression device: %1", device->errorString());
+            }
+        } else {
+            device = &imageFile;
+        }
+
         // Unmount volumes that belong to the selected target device
         // TODO: Check first if they are used and show warning
         // (problem: have to show request in the GUI thread and return reply back here)
@@ -156,7 +172,7 @@ void ImageWriter::writeImage()
             }
             else
             {
-                if ((readBytes = imageFile.read(static_cast<char*>(buffer), TRANSFER_BLOCK_SIZE)) <= 0)
+                if ((readBytes = device->read(static_cast<char*>(buffer), TRANSFER_BLOCK_SIZE)) <= 0)
                     break;
             }
             // Align the number of bytes to the sector size
@@ -172,17 +188,14 @@ void ImageWriter::writeImage()
             // For unknown reason, deviceFile.flush() does not work as intended here.
             fsync(deviceFile.handle());
 #endif
-
+            const int percent = (100 * imageFile.pos()) / imageFile.size();
             // Inform the GUI thread that next block was written
             // TODO: Make sure that when TRANSFER_BLOCK_SIZE is not a multiple of DEFAULT_UNIT
             // this still works or at least fails compilation
-            emit blockWritten(TRANSFER_BLOCK_SIZE / DEFAULT_UNIT);
+            emit progressChanged(percent);
 
 #if defined(Q_OS_LINUX)
-            KAuth::HelperSupport::progressStep(TRANSFER_BLOCK_SIZE / DEFAULT_UNIT); // FIXME why doesn't this work?
-            QVariantMap progressArgs;
-            progressArgs[QStringLiteral("progress")] = TRANSFER_BLOCK_SIZE / DEFAULT_UNIT;
-            KAuth::HelperSupport::progressStep(progressArgs); // used because above doesn't work
+            KAuth::HelperSupport::progressStep(percent);
 #endif
 
             // Check for the cancel request (using temporary variable to avoid multiple unlock calls in the code)
@@ -215,8 +228,9 @@ void ImageWriter::writeImage()
         }
         if (!zeroing)
         {
-            if (readBytes < 0)
-                throw i18n("Failed to read the image file:\n%1", imageFile.errorString());
+            if (readBytes < 0) {
+                throw i18n("Failed to read the image file:\n%1", device->errorString());
+            }
             imageFile.close();
         }
         deviceFile.close();
