@@ -1,19 +1,8 @@
 /*
- * Copyright 2017 Jonathan Riddell <jr@jriddell.org>
- * 
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+    SPDX-FileCopyrightText: 2017 Jonathan Riddell <jr@jriddell.org>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "imagewriter_helper.h"
 #include "isoimagewriter_debug.h"
@@ -21,7 +10,12 @@
 #include "usbdevice.h"
 
 #include <KLocalizedString>
+#include <kauth_version.h>
+#if KAUTH_VERSION >= QT_VERSION_CHECK(5, 92, 0)
+#include <KAuth/ActionReply>
+#else
 #include <KAuthActionReply>
+#endif
 
 #include <QProcess>
 #include <QDateTime>
