@@ -1,18 +1,7 @@
 /*
- * Copyright 2016 ROSA
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+    SPDX-FileCopyrightText: 2016 ROSA
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include <QApplication>
 #include <QTranslator>
@@ -22,6 +11,7 @@
 
 #include <KAboutData>
 #include <KLocalizedString>
+#include <KCrash>
 
 #include "common.h"
 #include "mainapplication.h"
@@ -42,6 +32,7 @@ int main(int argc, char *argv[])
 #endif
 
     MainApplication a(argc, argv);
+    KCrash::initialize();
 
     if (!ensureElevated())
         return 1;
