@@ -1,23 +1,13 @@
 /*
- * Copyright 2016 ROSA
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+    SPDX-FileCopyrightText: 2016 ROSA
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "mainapplication.h"
 
 #include <QStandardPaths>
 #include <QIcon>
+#include <QUrl>
 #include <KAboutData>
 #include <KLocalizedString>
 
@@ -42,8 +32,6 @@ MainApplication::MainApplication(int& argc, char** argv) :
     aboutData.addAuthor(i18n("Jonathan Riddell"), i18n("Author"), QStringLiteral("jr@jriddell.org"));
     m_Options.addOption(QCommandLineOption("lang", "", "language"));
     m_Options.addOption(QCommandLineOption("dir", "", "path"));
-    m_Options.addHelpOption();
-    m_Options.addVersionOption();
     aboutData.setupCommandLine(&m_Options);
     m_Options.process(arguments());
     aboutData.processCommandLine(&m_Options);
@@ -80,11 +68,11 @@ QString MainApplication::getInitialDir()
 }
 
 // Returns the fila path passed to the application as command-line parameter
-QString MainApplication::getInitialImage()
+QUrl MainApplication::getInitialImage()
 {
-    QStringList args = m_Options.positionalArguments();
-    if (args.size() > 0)
-        return args.at(0);
+    const QStringList args = m_Options.positionalArguments();
+    if (!args.isEmpty())
+        return QUrl::fromUserInput(args.first(), getInitialDir(), QUrl::AssumeLocalFile);
     else
-        return "";
+        return {};
 }
