@@ -1,23 +1,13 @@
 /*
- * Copyright 2019 Farid Boudedja <farid.boudedja@gmail.com>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+    SPDX-FileCopyrightText: 2019 Farid Boudedja <farid.boudedja@gmail.com>
+
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
 #include "mainwindow.h"
 #include "mainapplication.h"
 #include "common.h"
+#include "fetchisojob.h"
 #include "imagewriter.h"
 #include "isoverifier.h"
 #include "isoimagewriter_debug.h"
@@ -33,6 +23,7 @@
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QInputDialog>
+#include <QStandardPaths>
 #include <KFormat>
 #include <KIconLoader>
 #include <KPixmapSequence>
@@ -40,7 +31,7 @@
 
 MainWindow::MainWindow(QWidget *parent)
     : QMainWindow(parent),
-      m_lastOpenedDir(""),
+      m_lastOpenedDir(),
       m_isWriting(false),
       m_enumFlashDevicesWaiting(false),
       m_externalProgressBar(this)
@@ -51,19 +42,13 @@ MainWindow::MainWindow(QWidget *parent)
     // Set initial directory
     m_lastOpenedDir = mApp->getInitialDir();
     // Get path to ISO image from command line args (if supplied)
-    QString isoImagePath = mApp->getInitialImage();
-    if (!isoImagePath.isEmpty())
-    {
-        if (isoImagePath.left(7) == "file://")
-            isoImagePath = QUrl(isoImagePath).toLocalFile();
-
-        if (!isoImagePath.isEmpty())
-        {
-            isoImagePath = QDir(isoImagePath).absolutePath();
-            // Update the default open dir
-            m_lastOpenedDir = isoImagePath.left(isoImagePath.lastIndexOf('/'));
-            preprocessIsoImage(isoImagePath);
-        }
+    QUrl isoImagePath = mApp->getInitialImage();
+    if (isoImagePath.isLocalFile()) {
+        const QString path = isoImagePath.toLocalFile();
+        m_lastOpenedDir = path.left(path.lastIndexOf('/'));
+        preprocessIsoImage(path);
+    } else {
+        m_isoImageLineEdit->setText(isoImagePath.toString());
     }
 
     // Load the list of USB flash devices
@@ -90,7 +75,7 @@ void MainWindow::setupUi()
 {
     // Logo
     QLabel *logoLabel = new QLabel;
-    logoLabel->setPixmap(QIcon::fromTheme("drive-removable-media").pixmap(QSize(50, 50)));
+    logoLabel->setPixmap(KIconLoader::global()->loadIcon("drive-removable-media", KIconLoader::Desktop));
     logoLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
     QLabel *titleLabel = new QLabel;
@@ -125,6 +110,7 @@ QWidget* MainWindow::createFormWidget()
     m_isoImageLineEdit = new QLineEdit;
     m_isoImageLineEdit->setReadOnly(true);
     m_isoImageLineEdit->setPlaceholderText(i18n("Path to ISO image..."));
+    m_isoImageSizeLabel = new QLabel;
 
     QAction *openIsoImageAction = m_isoImageLineEdit->addAction(
         QIcon::fromTheme("folder-open"), QLineEdit::TrailingPosition);
@@ -133,7 +119,6 @@ QWidget* MainWindow::createFormWidget()
     m_usbDriveComboBox = new QComboBox;
 
     m_createButton = new QPushButton(i18n("Create"));
-    m_createButton->setEnabled(false);
     connect(m_createButton, &QPushButton::clicked, this, &MainWindow::showConfirmMessage);
 
     m_busyLabel = new QLabel;
@@ -148,9 +133,13 @@ QWidget* MainWindow::createFormWidget()
     footerBoxLayout->addWidget(m_busyLabel);
     footerBoxLayout->addWidget(m_createButton, 0, Qt::AlignRight);
 
+    QHBoxLayout *isoImageLayout = new QHBoxLayout;
+    isoImageLayout->addWidget(m_isoImageLineEdit);
+    isoImageLayout->addWidget(m_isoImageSizeLabel);
+
     QVBoxLayout *mainVBoxLayout = new QVBoxLayout;
     mainVBoxLayout->addWidget(new QLabel(i18n("Write this ISO image:")));
-    mainVBoxLayout->addWidget(m_isoImageLineEdit);
+    mainVBoxLayout->addLayout(isoImageLayout);
     mainVBoxLayout->addSpacing(5);
     mainVBoxLayout->addWidget(new QLabel(i18n("To this USB drive:")));
     mainVBoxLayout->addWidget(m_usbDriveComboBox);
@@ -170,19 +159,37 @@ QWidget* MainWindow::createConfirmWidget()
     iconLabel->setPixmap(QIcon::fromTheme("dialog-warning").pixmap(QSize(64, 64)));
     iconLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
-    QLabel *messageLabel = new QLabel(i18n("Everything on the USB drive will "
+    static const QString overwriteMessage = i18n("Everything on the USB drive will "
                                            "be overwritten."
-                                           "\n\nDo you want to continue?"));
+                                           "\n\nDo you want to continue?");
+    QLabel *messageLabel = new QLabel(overwriteMessage);
+    connect(this, &MainWindow::downloadProgressChanged, messageLabel, [messageLabel, iconLabel, this] {
+        iconLabel->setPixmap(QIcon::fromTheme("download").pixmap(QSize(64, 64)));
+        messageLabel->setText(i18n("Downloading %1...", m_fetchIso->fetchUrl().toDisplayString()));
+    });
+    connect(this, &MainWindow::verificationResult, messageLabel, [messageLabel, iconLabel] {
+        messageLabel->setText(overwriteMessage);
+        iconLabel->setPixmap(QIcon::fromTheme("dialog-warning").pixmap(QSize(64, 64)));
+    });
 
     QHBoxLayout *messageHBoxLayout = new QHBoxLayout;
     messageHBoxLayout->addWidget(iconLabel, 0, Qt::AlignTop);
     messageHBoxLayout->addWidget(messageLabel, 0, Qt::AlignTop);
 
+    QProgressBar *downloadProgressBar = new QProgressBar();
+    downloadProgressBar->setVisible(false);
+    downloadProgressBar->setMinimum(0);
+    downloadProgressBar->setMaximum(100);
+    connect(this, &MainWindow::downloadProgressChanged, downloadProgressBar, &QProgressBar::show);
+    connect(this, &MainWindow::downloadProgressChanged, downloadProgressBar, &QProgressBar::setValue);
+
     QPushButton *abortButton = new QPushButton(i18n("Abort"));
     connect(abortButton, &QPushButton::clicked, this, &MainWindow::hideWritingProgress);
 
     QPushButton *continueButton = new QPushButton(i18n("Continue"));
     connect(continueButton, &QPushButton::clicked, this, &MainWindow::writeIsoImage);
+    connect(this, &MainWindow::downloadProgressChanged, continueButton, [continueButton] { continueButton->setEnabled(false); });
+    connect(this, &MainWindow::verificationResult, continueButton, [continueButton] { continueButton->setEnabled(true); });
 
     QHBoxLayout *buttonsHBoxLayout = new QHBoxLayout;
     buttonsHBoxLayout->addWidget(abortButton, 0, Qt::AlignLeft);
@@ -190,6 +197,7 @@ QWidget* MainWindow::createConfirmWidget()
 
     QVBoxLayout *mainVBoxLayout = new QVBoxLayout;
     mainVBoxLayout->addLayout(messageHBoxLayout);
+    mainVBoxLayout->addWidget(downloadProgressBar);
     mainVBoxLayout->addLayout(buttonsHBoxLayout);
 
     QWidget *confirmWidget = new QWidget;
@@ -266,11 +274,12 @@ void MainWindow::preprocessIsoImage(const QString& isoImagePath)
 
     m_isoImageSize = file.size();
     m_isoImagePath = isoImagePath;
-    m_isoImageLineEdit->setText(QDir::toNativeSeparators(m_isoImagePath) + " ("
-                                + KFormat().formatByteSize(m_isoImageSize) + ")");
+    m_isoImageLineEdit->setText(QDir::toNativeSeparators(m_isoImagePath));
+    m_isoImageSizeLabel->setText(KFormat().formatByteSize(m_isoImageSize));
 
     file.close();
 
+#ifdef _USE_GPG
     // Verify ISO image
     m_busyLabel->setText(i18n("Verifying ISO image"));
     m_busyWidget->show();
@@ -292,6 +301,7 @@ void MainWindow::preprocessIsoImage(const QString& isoImagePath)
 
     isoVerifier->moveToThread(verifierThread);
     verifierThread->start();
+#endif
 
     // Enable the Write button (if there are USB flash disks present)
     m_createButton->setEnabled(m_usbDriveComboBox->count() > 0);
@@ -383,8 +393,8 @@ void MainWindow::writeToDevice(bool zeroing)
 
     m_job = action.execute();
 
-    connect(m_job, SIGNAL(percent(KJob*,ulong)), this, SLOT(progressStep(KJob*,ulong)), Qt::DirectConnection);
-    connect(m_job, SIGNAL(newData(QVariantMap)), this, SLOT(progressStep(QVariantMap)));
+    connect(m_job, &KAuth::ExecuteJob::percentChanged, this, &MainWindow::progressPercentUpdate);
+    connect(m_job, &KAuth::ExecuteJob::newData, this, &MainWindow::progressStep);
     connect(m_job, &KAuth::ExecuteJob::statusChanged, this, &MainWindow::statusChanged);
     connect(m_job, &KAuth::ExecuteJob::result, this, &MainWindow::finished);
 
@@ -404,7 +414,7 @@ void MainWindow::writeToDevice(bool zeroing)
     // Using DirectConnection because the thread does not read its own event queue until completion
     connect(m_cancelButton, &QPushButton::clicked, writer, &ImageWriter::cancelWriting, Qt::DirectConnection);
     // Each time a block is written, update the progress bar
-    connect(writer, &ImageWriter::blockWritten, this, &MainWindow::updateProgressBar);
+    connect(writer, &ImageWriter::progressChanged, this, &MainWindow::updateProgressBar);
     // Show the message about successful completion on success
     connect(writer, &ImageWriter::success, this, &MainWindow::showSuccessMessage);
     // Show error message if error is sent by the worker
@@ -417,7 +427,7 @@ void MainWindow::writeToDevice(bool zeroing)
     writerThread->start();
 #endif
 
-    showWritingProgress(alignNumberDiv((zeroing ? DEFAULT_UNIT : m_isoImageSize), DEFAULT_UNIT));
+    showWritingProgress();
 }
 
 void MainWindow::dragEnterEvent(QDragEnterEvent* event)
@@ -455,7 +465,7 @@ void MainWindow::dropEvent(QDropEvent* event)
                 if ((newLineIndexLF != -1) && (newLineIndexLF < newLineIndex))
                     newLineIndex = newLineIndexLF;
                 if (newLineIndex != -1)
-                    droppedFileName = droppedFileName.left(newLineIndex);
+                    droppedFileName.truncate(newLineIndex);
                 // Decode the file path from percent-encoding
                 QUrl url = QUrl::fromEncoded(droppedFileName);
                 if (url.isLocalFile())
@@ -463,11 +473,6 @@ void MainWindow::dropEvent(QDropEvent* event)
             }
         }
     }
-
-    if (!newImageFile.isEmpty()) {
-        // If something was really received update the information
-        preprocessIsoImage(newImageFile);
-    }
 }
 
 void MainWindow::closeEvent(QCloseEvent* event)
@@ -493,16 +498,43 @@ void MainWindow::addFlashDeviceCallback(void* cbParam, UsbDevice* device)
 
 void MainWindow::openIsoImage()
 {
-    const QString filter = i18n("Disk Images (%1)", QString("*.iso *.bin *.img"))
+    const QString filter = i18n("Disk Images (%1)", QString("*.iso *.bin *.img *.iso.gz *.iso.xz *.img.zstd *.img.gz *.img.zx *.img.zstd"))
         + ";;" + i18n("All Files (%1)", QString("*"));
-    QString isoImagePath = QFileDialog::getOpenFileName(this, "", m_lastOpenedDir, 
+    QUrl isoImageUrl = QFileDialog::getOpenFileUrl(this, i18n("Select image to flash"), QUrl::fromLocalFile(m_lastOpenedDir),
                                                         filter, nullptr,
                                                         QFileDialog::ReadOnly);
-    if (!isoImagePath.isEmpty())
-    {
-        m_lastOpenedDir = isoImagePath.left(isoImagePath.lastIndexOf('/'));
-        preprocessIsoImage(isoImagePath);
+    openUrl(isoImageUrl);
+}
+
+void MainWindow::openUrl(const QUrl& url)
+{
+    if (url.isEmpty()) {
+        return;
     }
+
+    if (url.isLocalFile()) {
+        const QString path = url.toLocalFile();
+        m_lastOpenedDir = path.left(path.lastIndexOf('/'));
+        preprocessIsoImage(path);
+        return;
+    } else {
+        m_isoImageLineEdit->setText(url.toString());
+    }
+
+    delete m_fetchIso;
+    m_fetchIso = new FetchIsoJob(this);
+    connect(m_fetchIso, &FetchIsoJob::downloadProgressChanged, this, &MainWindow::downloadProgressChanged);
+    connect(m_fetchIso, &FetchIsoJob::failed, this, &MainWindow::hideWritingProgress);
+    connect(m_fetchIso, &FetchIsoJob::finished, this, [this] (const QString &file) {
+        m_isoImagePath = file;
+        m_busySpinner->stop();
+        preprocessIsoImage(file);
+    });
+    m_busyLabel->setText(i18n("Downloading ISO image"));
+    m_busyWidget->show();
+    m_busySpinner->setSequence(KIconLoader::global()->loadPixmapSequence("process-working", KIconLoader::SizeSmallMedium));
+    m_busySpinner->start();
+    m_fetchIso->fetch(url);
 }
 
 void MainWindow::writeIsoImage()
@@ -513,8 +545,19 @@ void MainWindow::writeIsoImage()
     UsbDevice* selectedDevice = m_usbDriveComboBox->itemData(
         m_usbDriveComboBox->currentIndex()).value<UsbDevice*>();
 
-    if (m_isoImageSize > selectedDevice->m_Size)
-    {
+    if (selectedDevice->m_Size == 0) {
+        int warningReturn = QMessageBox::warning(
+            this,
+            i18n("Unknown Disk Size"),
+            i18n("The selected disk is of unknown size, please check the image will fit before writing.\n\n"
+                 "Image size: %1 (%2 b)",
+                 KFormat().formatByteSize(m_isoImageSize),
+                 m_isoImageSize),
+            QMessageBox::Ok | QMessageBox::Cancel);
+        if (warningReturn != QMessageBox::Ok) {
+            return;
+        }
+    } else if (m_isoImageSize > selectedDevice->m_Size) {
         QMessageBox::critical(
             this,
             i18n("Error"),
@@ -532,14 +575,13 @@ void MainWindow::writeIsoImage()
     writeToDevice(false);
 }
 
-void MainWindow::updateProgressBar(int increment)
+void MainWindow::updateProgressBar(int percent)
 {
-    int newValue = m_progressBar->value() + increment;
-    m_progressBar->setValue(newValue);
-    m_externalProgressBar.SetProgressValue(newValue);
+    m_progressBar->setValue(percent);
+    m_externalProgressBar.SetProgressValue(percent);
 }
 
-void MainWindow::showWritingProgress(int maxValue)
+void MainWindow::showWritingProgress()
 {
     m_isWriting = true;
 
@@ -548,11 +590,11 @@ void MainWindow::showWritingProgress(int maxValue)
 
     // Display and customize the progress bar part
     m_progressBar->setMinimum(0);
-    m_progressBar->setMaximum(maxValue);
+    m_progressBar->setMaximum(100);
     m_progressBar->setValue(0);
 
     // Expose the progress bar state to the OS
-    m_externalProgressBar.InitProgressBar(maxValue);
+    m_externalProgressBar.InitProgressBar(100);
 
     m_centralStackedWidget->setCurrentIndex(2);
 }
@@ -595,23 +637,30 @@ void MainWindow::showSuccessMessage()
 
 void MainWindow::showConfirmMessage()
 {
+    openUrl(QUrl::fromUserInput(m_isoImageLineEdit->text(), {}, QUrl::AssumeLocalFile));
+
     // Do not accept dropped files
     setAcceptDrops(false);
 
     m_centralStackedWidget->setCurrentIndex(1);
 }
 
-void MainWindow::showIsoVerificationResult(const bool &isIsoValid, const QString &error)
+void MainWindow::showIsoVerificationResult(IsoVerifier::VerifyResult verify, const QString &error)
 {
-    if (isIsoValid) {
+    if (verify == IsoVerifier::VerifyResult::Successful) {
         m_busyLabel->setText(i18n("The ISO image is valid"));
         m_busySpinner->setSequence(KIconLoader::global()->loadPixmapSequence("checkmark", KIconLoader::SizeSmallMedium));
     } else {
-        m_busyLabel->setText(i18n("Could not verify ISO image"));
         m_busySpinner->setSequence(KIconLoader::global()->loadPixmapSequence("error", KIconLoader::SizeSmallMedium));
 
-        QMessageBox::warning(this, i18n("ISO Verification failed"), error);
+        if (verify == IsoVerifier::VerifyResult::KeyNotFound) {
+            m_busyLabel->setText(i18n("Could not find the key to verify the ISO image"));
+        } else if (verify == IsoVerifier::VerifyResult::Failed) {
+            m_busyLabel->setText(i18n("Could not verify ISO image"));
+            QMessageBox::warning(this, i18n("ISO Verification failed"), error);
+        }
     }
+    Q_EMIT verificationResult(verify == IsoVerifier::VerifyResult::Successful);
 }
 
 #if defined(Q_OS_LINUX)
@@ -621,18 +670,15 @@ void MainWindow::cancelWriting() {
     qCDebug(ISOIMAGEWRITER_LOG) << "cancelWriting() done";
 }
 
-void MainWindow::progressStep(KJob* job, unsigned long step) {
+void MainWindow::progressPercentUpdate(KJob* job, unsigned long step) {
     Q_UNUSED(job)
     qCDebug(ISOIMAGEWRITER_LOG) << "progressStep %() " << step;
     updateProgressBar(step);
 }
 
 void MainWindow::progressStep(const QVariantMap & data) {
-    qCDebug(ISOIMAGEWRITER_LOG) << "progressStep(QVariantMap) ";// << step;
-    if (data[QStringLiteral("progress")].isValid()) {
-        int step = data[QStringLiteral("progress")].toInt();
-        updateProgressBar(step);
-    } else if (data[QStringLiteral("error")].isValid()) {
+    qCDebug(ISOIMAGEWRITER_LOG) << "progressStep(QVariantMap)" << data;// << step;
+    if (data[QStringLiteral("error")].isValid()) {
         showErrorMessage(data[QStringLiteral("error")].toString());
     } else if (data[QStringLiteral("success")].isValid()) {
         showSuccessMessage();
